@@ -32,6 +32,8 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+import textwrap
+from phonopy.harmonic.dynmat_to_fc import DynmatToForceConstants
 import numpy as np
 from phonopy.structure.cells import get_reduced_bases
 
@@ -94,14 +96,16 @@ class DynamicalMatrix(object):
         self._decimals = decimals
         self._symprec = symprec
 
+        itemsize = self._force_constants.itemsize
+        self._dtype_complex = ("c%d" % (itemsize * 2))
+
         self._p2s_map = primitive.get_primitive_to_supercell_map()
         self._s2p_map = primitive.get_supercell_to_primitive_map()
         p2p_map = primitive.get_primitive_to_primitive_map()
         self._p2p_map = [p2p_map[self._s2p_map[i]]
                          for i in range(len(self._s2p_map))]
-        self._smallest_vectors, self._multiplicity = \
-            get_smallest_vectors(supercell, primitive, symprec)
-        self._mass = self._pcell.get_masses()
+        (self._smallest_vectors,
+         self._multiplicity) = primitive.get_smallest_vectors()
         # Non analytical term correction
         self._nac = False
 
@@ -140,74 +144,15 @@ class DynamicalMatrix(object):
         else:
             return dm.round(decimals=self._decimals)
 
-    def set_dynamical_matrix(self, q, verbose=False):
+    def set_dynamical_matrix(self, q):
+        self._set_dynamical_matrix(q)
+
+    def _set_dynamical_matrix(self, q):
         try:
             import phonopy._phonopy as phonoc
             self._set_c_dynamical_matrix(q)
         except ImportError:
-            self._set_py_dynamical_matrix(q, verbose=verbose)
-
-    def _set_py_dynamical_matrix(self,
-                                 q,
-                                 verbose=False):
-        fc = self._force_constants
-        vecs = self._smallest_vectors
-        multiplicity = self._multiplicity
-        num_atom = len(self._p2s_map)
-        dm = np.zeros((3 * num_atom, 3 * num_atom), dtype=complex)
-
-        for i, s_i in enumerate(self._p2s_map):
-            for j, s_j in enumerate(self._p2s_map):
-                mass = np.sqrt(self._mass[i] * self._mass[j])
-                dm_local = np.zeros((3, 3), dtype=complex)
-                # Sum in lattice points                
-                for k in range(self._scell.get_number_of_atoms()): 
-                    if s_j == self._s2p_map[k]:
-                        multi = multiplicity[k][i]
-                        phase = []
-                        for l in range(multi):
-                            vec = vecs[k][i][l]
-                            phase.append(np.vdot(vec, q) * 2j * np.pi)
-                        phase_factor = np.exp(phase).sum()
-                        dm_local += fc[s_i, k] * phase_factor / mass / multi
-
-                dm[(i*3):(i*3+3), (j*3):(j*3+3)] += dm_local
-
-        # Impose Hermisian condition
-        self._dynamical_matrix = (dm + dm.conj().transpose()) / 2 
-
-        if verbose:
-            self._dynamical_matrix_log()
-
-    def _dynamical_matrix_log(self):
-        dm = self._dynamical_matrix
-        for i in range(dm.shape[0] // 3):
-            for j in range(dm.shape[0] // 3):
-                dm_local = dm[(i*3):(i*3+3), (j*3):(j*3+3)]
-                for vec in dm_local:
-                    re = vec.real
-                    im = vec.imag
-                    print("dynamical matrix(%3d - %3d) "
-                          "%10.5f %10.5f %10.5f %10.5f %10.5f %10.5f" % 
-                          (i+1, j+1, re[0], im[0], re[1], im[1], re[2], im[2]))
-                print('')
-
-    def _smallest_vectors_log(self):
-        r = self._smallest_vectors
-        m = self._multiplicity
-
-        print("#%4s %4s %4s %4s %4s %10s" % 
-              ("p_i", "p_j", "s_i", "s_j", "mult", "dist"))
-        for p_i, s_i in enumerate(self._p2s_map): # run in primitive
-            for s_j in range(r.shape[0]): # run in supercell
-                for tmp_p_j, tmp_s_j in enumerate(self._p2s_map):
-                    if self._s2p_map[s_j] == tmp_s_j:
-                        p_j = tmp_p_j
-                for k in range(m[s_j][p_i]):
-                    print(" %4d %4d %4d %4d %4d %10.5f" %
-                          (p_i+1, p_j+1, s_i+1, s_j+1, m[s_j][p_i],
-                           np.linalg.norm(np.dot(r[s_j][p_i][k],
-                                                 self._pcell.get_cell()))))
+            self._set_py_dynamical_matrix(q)
 
     def _set_c_dynamical_matrix(self, q):
         import phonopy._phonopy as phonoc
@@ -230,6 +175,34 @@ class DynamicalMatrix(object):
                                 self._p2s_map)
         self._dynamical_matrix = dm
 
+    def _set_py_dynamical_matrix(self, q):
+        fc = self._force_constants
+        vecs = self._smallest_vectors
+        multiplicity = self._multiplicity
+        num_atom = len(self._p2s_map)
+        dm = np.zeros((3 * num_atom, 3 * num_atom), dtype=self._dtype_complex)
+        mass = self._pcell.get_masses()
+
+        for i, s_i in enumerate(self._p2s_map):
+            for j, s_j in enumerate(self._p2s_map):
+                sqrt_mm = np.sqrt(mass[i] * mass[j])
+                dm_local = np.zeros((3, 3), dtype=self._dtype_complex)
+                # Sum in lattice points                
+                for k in range(self._scell.get_number_of_atoms()): 
+                    if s_j == self._s2p_map[k]:
+                        multi = multiplicity[k][i]
+                        phase = []
+                        for l in range(multi):
+                            vec = vecs[k][i][l]
+                            phase.append(np.vdot(vec, q) * 2j * np.pi)
+                        phase_factor = np.exp(phase).sum()
+                        dm_local += fc[s_i, k] * phase_factor / sqrt_mm / multi
+
+                dm[(i*3):(i*3+3), (j*3):(j*3+3)] += dm_local
+
+        # Impose Hermisian condition
+        self._dynamical_matrix = (dm + dm.conj().transpose()) / 2 
+
 # Non analytical term correction (NAC)
 # Call this when NAC is required instead of DynamicalMatrix
 class DynamicalMatrixNAC(DynamicalMatrix):
@@ -249,6 +222,11 @@ class DynamicalMatrixNAC(DynamicalMatrix):
                                  symprec=1e-5)
         self._bare_force_constants = self._force_constants.copy()
 
+        # For method == 'gonze'
+        self._Gonze_force_constants = None
+        self._G_vec_list = None
+        self._G_cutoff = None
+
         self._nac = True
         if nac_params is not None:
             self.set_nac_params(nac_params)
@@ -267,69 +245,69 @@ class DynamicalMatrixNAC(DynamicalMatrix):
         self._unit_conversion = nac_params['factor']
         self._dielectric = np.array(nac_params['dielectric'],
                                     dtype='double', order='C')
-
-    def set_dynamical_matrix(self, q_red, q_direction=None, verbose=False):
-        num_atom = self._pcell.get_number_of_atoms()
-
-        if q_direction is None:
-            q = np.dot(q_red, np.linalg.inv(self._pcell.get_cell()).T)
+        if 'method' not in nac_params:
+            self._method = 'wang'
+        elif nac_params['method'] == 'gonze':
+            self._method = 'gonze'
         else:
-            q = np.dot(q_direction, np.linalg.inv(self._pcell.get_cell()).T)
+            self._method = 'wang'
 
-        if (q_direction is None and np.abs(q).sum() < self._symprec) or \
-                ((q_direction is not None) and
-                 np.abs(q_direction).sum() < self._symprec):
+        if self._method == 'gonze':
+            if 'G_cutoff' in nac_params:
+                self._G_cutoff = nac_params['G_cutoff']
+            else:
+                self._G_cutoff = 4
+            rec_lat = np.linalg.inv(self._pcell.get_cell()) # column vectors
+            G_vec_list = self._get_G_list(rec_lat)
+            G_norm = np.sqrt(((G_vec_list) ** 2).sum(axis=1))
+            self._G_list = G_vec_list[G_norm < self._G_cutoff]
+            print("G-cutoff distance: %f, number of G-points: %d" %
+                  (self._G_cutoff, len(self._G_list)))
+            print(self._G_list)
+            self._set_Gonze_force_constants()
+            self._Gonze_count = 0
+
+    def set_dynamical_matrix(self, q_red, q_direction=None):
+        rec_lat = np.linalg.inv(self._pcell.get_cell()) # column vectors
+        if q_direction is None:
+            q_norm = np.linalg.norm(np.dot(q_red, rec_lat.T))
+        else:
+            q_norm = np.linalg.norm(np.dot(q_direction, rec_lat.T))
+
+        if q_norm < self._symprec:
             self._force_constants = self._bare_force_constants.copy()
-            DynamicalMatrix.set_dynamical_matrix(self, q_red, verbose)
+            self._set_dynamical_matrix(q_red)
             return False
     
-        volume = self._pcell.get_volume()
-        constant = (self._unit_conversion * 4.0 * np.pi / volume
-                    / np.dot(q, np.dot(self._dielectric, q)))
+        if self._method == 'wang':
+            self._set_Wang_dynamical_matrix(q_red, q_direction)
+        else:
+            self._set_Gonze_dynamical_matrix(q_red, q_direction)
 
+    def _set_Wang_dynamical_matrix(self, q_red, q_direction):
         # Wang method (J. Phys.: Condens. Matter 22 (2010) 202201)
-        import phonopy._phonopy as phonoc
+        rec_lat = np.linalg.inv(self._pcell.get_cell()) # column vectors
+        if q_direction is None:
+            q = np.dot(q_red, rec_lat.T)
+        else:
+            q = np.dot(q_direction, rec_lat.T)
+
+        constant = self._get_constant_factor(q,
+                                             self._dielectric,
+                                             self._pcell.get_volume(),
+                                             self._unit_conversion)
         try:
             import phonopy._phonopy as phonoc
-            self._set_c_nac_dynamical_matrix(q_red, q, constant)
+            self._set_c_Wang_dynamical_matrix(q_red, q, constant)
         except ImportError:
+            num_atom = self._pcell.get_number_of_atoms()
             fc = self._bare_force_constants.copy()
-            nac_q = np.zeros((num_atom, num_atom, 3, 3), dtype='double')
-            for i in range(num_atom):
-                A_i = np.dot(q, self._born[i])
-                for j in range(num_atom):
-                    A_j = np.dot(q, self._born[j])
-                    nac_q[i, j] = np.outer(A_i, A_j) * constant
-            self._set_NAC_force_constants(fc, nac_q)
+            nac_q = self._get_charge_sum(num_atom, q, self._born) * constant
+            self._set_py_Wang_force_constants(fc, nac_q)
             self._force_constants = fc
-            DynamicalMatrix.set_dynamical_matrix(self, q_red, verbose)
+            self._set_dynamical_matrix(q_red)
 
-    def _set_NAC_force_constants(self, fc, nac_q):
-        N = (self._scell.get_number_of_atoms() //
-             self._pcell.get_number_of_atoms())
-        for s1 in range(self._scell.get_number_of_atoms()):
-            # This if-statement is the trick.
-            # In contructing dynamical matrix in phonopy
-            # fc of left indices with s1 == self._s2p_map[ s1 ] are
-            # only used.
-            if s1 != self._s2p_map[s1]:
-                continue
-            p1 = self._p2p_map[s1]
-            for s2 in range(self._scell.get_number_of_atoms()):            
-                p2 = self._p2p_map[s2]
-                fc[s1, s2] += nac_q[p1, p2] / N
-
-    def _get_charge_sum(self, num_atom, q):
-        charge_sum = np.zeros((num_atom, num_atom, 3, 3), dtype='double')
-        for i in range(num_atom):
-            for j in range(num_atom):
-                for a in (0, 1, 2):
-                    for b in (0, 1, 2):
-                        charge_sum[i, j, a, b] = \
-                            np.dot(q, self._born[i, :, a]) * np.dot(q, self._born[j, :, b])
-        return charge_sum
-
-    def _set_c_nac_dynamical_matrix(self, q_red, q, factor):
+    def _set_c_Wang_dynamical_matrix(self, q_red, q, factor):
         import phonopy._phonopy as phonoc
 
         fc = self._bare_force_constants.copy()
@@ -353,6 +331,165 @@ class DynamicalMatrixNAC(DynamicalMatrix):
                                     factor)
         self._dynamical_matrix = dm
 
+    def _set_py_Wang_force_constants(self, fc, nac_q):
+        N = (self._scell.get_number_of_atoms() //
+             self._pcell.get_number_of_atoms())
+        for s1 in range(self._scell.get_number_of_atoms()):
+            # This if-statement is the trick.
+            # In contructing dynamical matrix in phonopy
+            # fc of left indices with s1 == self._s2p_map[ s1 ] are
+            # only used.
+            if s1 != self._s2p_map[s1]:
+                continue
+            p1 = self._p2p_map[s1]
+            for s2 in range(self._scell.get_number_of_atoms()):            
+                p2 = self._p2p_map[s2]
+                fc[s1, s2] += nac_q[p1, p2] / N
+
+    def _set_Gonze_dynamical_matrix(self, q_red, q_direction):
+        print("%d %s" % (self._Gonze_count + 1, q_red))
+        self._Gonze_count += 1
+        self._force_constants = self._Gonze_force_constants
+        self._set_dynamical_matrix(q_red)
+        dm_dd = self._get_Gonze_dipole_dipole(q_red, q_direction)
+        self._dynamical_matrix += dm_dd
+
+    def _set_Gonze_force_constants(self):
+        d2f = DynmatToForceConstants(self._pcell,
+                                     self._scell,
+                                     symprec=self._symprec)
+        self._force_constants = self._bare_force_constants
+        dynmat = []
+        num_q = len(d2f.get_commensurate_points())
+        for i, q_red in enumerate(d2f.get_commensurate_points()):
+            print("%d/%d %s" % (i + 1, num_q, q_red))
+            self._set_dynamical_matrix(q_red)
+            dm_dd = self._get_Gonze_dipole_dipole(q_red, None)
+            self._dynamical_matrix -= dm_dd
+            dynmat.append(self._dynamical_matrix)
+        d2f.set_dynamical_matrices(dynmat=dynmat)
+        d2f.run()
+        self._Gonze_force_constants = d2f.get_force_constants()
+
+    def _get_Gonze_dipole_dipole(self, q_red, q_direction):
+        rec_lat = np.linalg.inv(self._pcell.get_cell()) # column vectors
+        q = np.dot(q_red, rec_lat.T)
+        # K_norm = np.sqrt(((self._G_vec_list + q) ** 2).sum(axis=1))
+        # K_list = self._G_vec_list[K_norm < self._G_cutoff] + q
+        K_list = self._G_list + q
+        if q_direction is None:
+            q_dir_cart = None
+        else:
+            q_dir_cart = np.array(np.dot(q_direction, rec_lat.T),
+                                  dtype='double')
+        
+        try:
+            import phonopy._phonopy as phonoc
+            C = self._get_c_dipole_dipole(K_list, q, q_dir_cart)
+        except ImportError:
+            C = self._get_py_dipole_dipole(K_list, q, q_dir_cart)
+
+        mass = self._pcell.get_masses()
+        num_atom = self._pcell.get_number_of_atoms()
+        pos = self._pcell.get_positions()
+        for i in range(num_atom):
+            dpos = pos - pos[i]
+            phase_factor = np.exp(2j * np.pi * np.dot(dpos, q))
+            # phase_factor = np.ones(len(dpos))
+            for j in range(num_atom):
+                C[i, :, j, :] *= phase_factor[j] / np.sqrt(mass[i] * mass[j])
+
+        C_dd = C.reshape(num_atom * 3, num_atom * 3)
+                
+        return C_dd
+
+    def _get_c_dipole_dipole(self, K_list, q, q_dir_cart):
+        import phonopy._phonopy as phonoc
+
+        pos = self._pcell.get_positions()
+        num_atom = self._pcell.get_number_of_atoms()
+        volume = self._pcell.get_volume()
+        C = np.zeros((num_atom, 3, num_atom, 3),
+                     dtype=self._dtype_complex, order='C')
+
+        phonoc.dipole_dipole(C.view(dtype='double'),
+                             np.array(K_list, dtype='double', order='C'),
+                             q,
+                             q_dir_cart,
+                             self._born,
+                             self._dielectric,
+                             np.array(pos, dtype='double', order='C'),
+                             self._unit_conversion * 4.0 * np.pi / volume,
+                             self._symprec)
+        return C
+
+    def _get_py_dipole_dipole(self, K_list, q, q_dir_cart):
+        pos = self._pcell.get_positions()
+        num_atom = self._pcell.get_number_of_atoms()
+        volume = self._pcell.get_volume()
+        C = np.zeros((num_atom, 3, num_atom, 3),
+                     dtype=self._dtype_complex, order='C')
+
+        for q_K in K_list:
+            if np.linalg.norm(q_K) < self._symprec:
+                if q_dir_cart is None:
+                    continue
+                else:
+                    dq_K = q_dir_cart
+            else:
+                dq_K = q_K
+
+            Z_mat = (self._get_charge_sum(num_atom, dq_K, self._born) *
+                     self._get_constant_factor(dq_K,
+                                               self._dielectric,
+                                               volume,      
+                                               self._unit_conversion))
+            for i in range(num_atom):
+                dpos = - pos + pos[i]
+                phase_factor = np.exp(2j * np.pi * np.dot(dpos, q_K))
+                for j in range(num_atom):
+                    C[i, :,  j, :] += Z_mat[i, j] * phase_factor[j]
+
+        for q_K in K_list:
+            q_G = q_K - q
+            if np.linalg.norm(q_G) < self._symprec:
+                continue
+            Z_mat = (self._get_charge_sum(num_atom, q_G, self._born) *
+                     self._get_constant_factor(q_G,
+                                               self._dielectric,
+                                               volume,
+                                               self._unit_conversion))
+            for i in range(num_atom):
+                C_i = np.zeros((3, 3), dtype=self._dtype_complex, order='C')
+                dpos = - pos + pos[i]
+                phase_factor = np.exp(2j * np.pi * np.dot(dpos, q_G))
+                for j in range(num_atom):
+                    C_i += Z_mat[i, j] * phase_factor[j]
+                C[i, :,  i, :] -= C_i
+
+        return C
+
+    def _get_G_list(self, rec_lat, g_rad=100):
+        # g_rad must be greater than 0 for broadcasting.
+        n = g_rad * 2 + 1
+        G = np.zeros((n ** 3, 3), dtype='double', order='C')
+        pts = np.arange(-g_rad, g_rad + 1)
+        grid = np.meshgrid(pts, pts, pts)
+        for i in range(3):
+            G[:, i] = grid[i].ravel()
+        return np.dot(G, rec_lat.T)
+
+    def _get_charge_sum(self, num_atom, q, born):
+        nac_q = np.zeros((num_atom, num_atom, 3, 3), dtype='double', order='C')
+        A = np.dot(q, born)
+        for i in range(num_atom):
+            for j in range(num_atom):
+                nac_q[i, j] = np.outer(A[i], A[j])
+        return nac_q
+
+    def _get_constant_factor(self, q, dielectric, volume, unit_conversion):
+        return (unit_conversion * 4.0 * np.pi / volume /
+                np.dot(q.T, np.dot(dielectric, q)))
 
 # Helper methods
 def get_equivalent_smallest_vectors(atom_number_supercell,
@@ -360,70 +497,42 @@ def get_equivalent_smallest_vectors(atom_number_supercell,
                                     supercell,
                                     primitive_lattice,
                                     symprec):
-    reduced_bases = get_reduced_bases(supercell.get_cell(), symprec)
-    positions = np.dot(supercell.get_positions(), np.linalg.inv(reduced_bases))
+    from phonopy.structure.cells import get_equivalent_smallest_vectors as func
 
-    # Atomic positions are confined into the lattice made of reduced bases.
-    positions -= np.rint(positions)
+    print("")
+    print("****************************** warning "
+          "******************************")
+    msg = textwrap.wrap(
+        "phonopy.harmonic.dynamical_matrix.get_equivalent_smallest_vectors "
+        "was moved to phonopy.structure.cells.get_equivalent_smallest_vectors. "
+        "The alias, phonopy.harmonic.dynamical_matrix.get_equivalent_smallest_"
+        "vectors will be removed at some future version.\n")
+    print("\n".join(msg))
+    print("***************************************"
+          "******************************")
+    print("")
 
-    p_pos = positions[atom_number_primitive]
-    s_pos = positions[atom_number_supercell]
-    
-    # The vector arrow is from the atom in primitive to
-    # the atom in supercell cell plus a supercell lattice
-    # point. This is related to determine the phase
-    # convension when building dynamical matrix.
-    supercell_vectors = np.array([
-        [i, j, k] for i in (-1, 0, 1)
-                  for j in (-1, 0, 1)
-                  for k in (-1, 0, 1)
-    ])
-    
-    differences = s_pos + supercell_vectors - p_pos
-    distances = np.sqrt((np.dot(differences, reduced_bases) ** 2).sum(axis=1))
-    minimum = min(distances)
-    smallest_vectors = []
-    for i in range(27):
-        if abs(minimum - distances[i]) < symprec:
-            relative_scale = np.dot(reduced_bases,
-                                    np.linalg.inv(primitive_lattice))
-            smallest_vectors.append(np.dot(differences[i], relative_scale))
-            
-    return smallest_vectors
+    return func(atom_number_supercell,
+                atom_number_primitive,
+                supercell,
+                primitive_lattice,
+                symprec)
 
 def get_smallest_vectors(supercell, primitive, symprec):
-    """
-    shortest_vectors:
+    from phonopy.structure.cells import get_smallest_vectors as func
 
-      Shortest vectors from an atom in primitive cell to an atom in
-      supercell in the fractional coordinates. If an atom in supercell
-      is on the border centered at an atom in primitive and there are
-      multiple vectors that have the same distance and different
-      directions, several shortest vectors are stored. The
-      multiplicity is stored in another array, "multiplicity".
-      [atom_super, atom_primitive, multiple-vectors, 3]
-      
-    multiplicity:
-      Number of multiple shortest vectors (third index of "shortest_vectors")
-      [atom_super, atom_primitive]
-    """
-
-    p2s_map = primitive.get_primitive_to_supercell_map()
-    size_super = supercell.get_number_of_atoms()
-    size_prim = primitive.get_number_of_atoms()
-    shortest_vectors = np.zeros((size_super, size_prim, 27, 3), dtype='double')
-    multiplicity = np.zeros((size_super, size_prim), dtype='intc')
-
-    for i in range(size_super): # run in supercell
-        for j, s_j in enumerate(p2s_map): # run in primitive
-            vectors = get_equivalent_smallest_vectors(i,
-                                                      s_j,
-                                                      supercell, 
-                                                      primitive.get_cell(),
-                                                      symprec)
-            multiplicity[i][j] = len(vectors)
-            for k, elem in enumerate(vectors):
-                shortest_vectors[i][j][k] = elem
-
-    return shortest_vectors, multiplicity
+    print("")
+    print("****************************** warning "
+          "******************************")
+    msg = textwrap.wrap(
+        "phonopy.harmonic.dynamical_matrix.get_smallest_vectors "
+        "was moved to phonopy.structure.cells.get_smallest_vectors. "
+        "The alias, phonopy.harmonic.dynamical_matrix.get_smallest_vectors, "
+        "will be removed at some future version.")
+    print("\n".join(msg))
+    print("***************************************"
+          "******************************")
+    print("")
+    
+    return func(supercell, primitive, symprec)
 
